#[macro_use]
extern crate lazy_static;
extern crate rusty_dsp;

use rusty_dsp::multiply::MultiplyConst;
use rusty_dsp::sinks::SoundCardSink;
use rusty_dsp::sources::VFOSource;

mod baudot;
mod bitgenerator;
mod bitupsampler;
mod symbolgenerator;

fn main() {
    let data = "      kaufen sie jede woche vier gute bequeme pelze xy 1234567890 ";

    let encoder = baudot::Encoder::from(data.to_ascii_uppercase().into_bytes().into_iter());
    let bitgenerator = bitgenerator::BitGenerator::from(encoder);
    let bitresampler = bitupsampler::BitUpsampler::from(bitgenerator, 48000.0, 45.45);
    let symbolgenerator = symbolgenerator::SymbolGenerator::from(bitresampler, 1955.0, 2125.0);
    let source = VFOSource::new(symbolgenerator, 48000.0);
    let mult = MultiplyConst::from(source, 0.3);
    let mut sink = SoundCardSink::from(mult, 48000);

    sink.start().unwrap();
}
